/***************************************************************************
  Code for Physics 524 week 4 exercises studying I2C and SPI communications.
  
  This is basewd on Adafruit's example code, meant to exercise the Adafruit BME680 Breakout. See http://www.adafruit.com/products/3660

  Original code was written by Limor Fried & Kevin Townsend for Adafruit Industries, modified by George Gollin, University of Illinois, September 2023.
  
  Adafruit asks that the following text be included in any redistibuion of
  their code:

  This is a library for the BME680 gas, humidity, temperature & pressure sensor

  Designed specifically to work with the Adafruit BME680 Breakout
  ----> http://www.adafruit.com/products/3660

  These sensors use I2C or SPI to communicate, 2 or 4 pins are required
  to interface.

  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing products
  from Adafruit!

  Written by Limor Fried & Kevin Townsend for Adafruit Industries.
  BSD license, all text above must be included in any redistribution
 ***************************************************************************/

#include <Wire.h>
#include <SPI.h>
#include <Adafruit_Sensor.h>
#include "Adafruit_BME680.h"

// declare whether we'd like to talk to the I2C BME680
// or the SPI BME680. I have put both on a breadboard.

// #define DO_I2C true
#define DO_I2C false

/*
#define BME_SCK 13
#define BME_MISO 12
#define BME_MOSI 11
#define BME_CS 10
*/

// MEGA 2560 SPI pins...
#define BME_SCK 52
#define BME_MISO 50
#define BME_MOSI 51
#define BME_CS 53

#define SEALEVELPRESSURE_HPA (1013.25)

#if DO_I2C
  // I2C: SDA is pin 20, SCL is pin 21.
  Adafruit_BME680 bme; 
#else
  // hardware SPI
  Adafruit_BME680 bme(BME_CS); 
#endif

void setup() {
  // open the line to the serial monitor
  Serial.begin(115200);

  // wait for it to be ready
  while (!Serial);

  // print a few blank lines
  Serial.print("\n\n\n");

// message to the user:
#if DO_I2C
  Serial.println(F("BME680 test using I2C"));
#else
  // hardware SPI
  Serial.println(F("BME680 test using SPI"));
#endif

  if (!bme.begin()) {
    Serial.println("Could not find a valid BME680 sensor, check wiring!");
    while (1);
  }

  // Set up oversampling and filter initialization
  bme.setTemperatureOversampling(BME680_OS_8X);
  bme.setHumidityOversampling(BME680_OS_2X);
  bme.setPressureOversampling(BME680_OS_4X);
  bme.setIIRFilterSize(BME680_FILTER_SIZE_3);
  bme.setGasHeater(320, 150); // 320*C for 150 ms
}

void loop() {
  if (! bme.performReading()) {
    Serial.println("Failed to perform reading :(");
    return;
  }
  Serial.print("Temperature = ");
  Serial.print(bme.temperature);
  Serial.println(" *C");

  Serial.print("Pressure = ");
  Serial.print(bme.pressure / 100.0);
  Serial.println(" hPa");

  Serial.print("Humidity = ");
  Serial.print(bme.humidity);
  Serial.println(" %");

  Serial.print("Gas = ");
  Serial.print(bme.gas_resistance / 1000.0);
  Serial.println(" KOhms");

  Serial.print("Approx. Altitude = ");
  Serial.print(bme.readAltitude(SEALEVELPRESSURE_HPA));
  Serial.println(" m");

  Serial.println();
  delay(1000);
}